#define PANGO_ENABLE_BACKEND

#include <pango/pango.h>
#include <pango/pangoxft.h>
#include <pango/pangofc-decoder.h>
#include <pango/pangofc-fontmap.h>
#include <pango/pangofc-font.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>

#include "decoder.h"

PangoContext *pcontext;
PangoFontDescription *desc;
PangoFcFontMap *map;
PangoLayout *layout;
GtkWidget *window;
GtkWidget *area;
GdkGC     *gc;

PangoFcDecoderTest *decoder;

gboolean
expose_cb(GtkWidget *widget, GdkEventExpose *event);

gboolean
delete_cb(GtkWidget *widget, GdkEventAny *event);

void
prepare_cb (const gchar    *fontname,
	    FcPattern      *pattern);

void
coverage_cb (const gchar    *fontname,
	     FcChar32        from,
	     FcChar32       *to,
	     PangoFcFont    *font);

void
render_cb(const gchar    *fontname,
	  gunichar        from,
	  gunichar       *to,
	  PangoFcFont    *font);

PangoFcDecoder *
factory_callback (FcPattern *pattern, gpointer user_data);

int
main(int argc, char **argv)
{
  gtk_init(&argc, &argv);

  /* pango stuff */

  pcontext = pango_xft_get_context(GDK_DISPLAY(), 0);

  desc = pango_font_description_new();

  pango_font_description_set_family(desc, "Luxi Sans");
  pango_font_description_set_size(desc, 12 * PANGO_SCALE);

  pango_context_set_font_description(pcontext, desc);

  layout = pango_layout_new(pcontext);

  pango_layout_set_text(layout, "abcdefghijklmnopqrstuvwxyz", 26);

  /* gtk stuff */

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  area = gtk_drawing_area_new();

  gtk_container_add(GTK_CONTAINER(window), area);

  g_signal_connect(G_OBJECT(window), "delete_event",
		   G_CALLBACK(delete_cb), NULL);

  g_signal_connect(G_OBJECT(area), "expose_event", G_CALLBACK(expose_cb),
		   NULL);

  gtk_widget_show_all(window);

  gc = gdk_gc_new(area->window);

  /* get a copy of the xft font map and register a callback */
  map = PANGO_FC_FONT_MAP(pango_xft_get_font_map(GDK_DISPLAY(), 0));

  FcPattern *pattern = FcPatternCreate();
  FcPatternAddString(pattern, FC_FAMILY, (FcChar8 *)"Luxi Sans");

  pango_fc_font_map_add_decoder_factory (map, factory_callback, NULL);

  gtk_main();

  return 0;
}

gboolean
expose_cb(GtkWidget *widget, GdkEventExpose *event)
{
  gdk_draw_layout(area->window, gc, 2, 50, layout);
  return TRUE;
}

gboolean
delete_cb(GtkWidget *widget, GdkEventAny *event)
{
  printf("delete_cb\n");
  gtk_main_quit();
  return TRUE;
}

void
prepare_cb (const gchar    *fontname,
	    FcPattern      *pattern)
{
  printf("prepare_cb\n");
}

void
coverage_cb (const gchar    *fontname,
	     FcChar32        from,
	     FcChar32       *to,
	     PangoFcFont    *font)
{
  printf("coverage_cb 0x%x\n", from);
  *to = from;
}

void
render_cb(const gchar    *fontname,
	  gunichar        from,
	  PangoGlyph     *to,
	  PangoFcFont    *font)
{
  printf("render_cb 0x%x ", from);
  FcPattern *pattern = font->font_pattern;

  if (from + 13 > 0x7a) {
    from -= 13;
  }
  else {
    from += 13;
  }

  printf("0x%x\n", from);
   

  XftFont *xft_font;

  xft_font = XftFontOpenPattern(GDK_DISPLAY(), FcPatternDuplicate(pattern));

  *to = XftCharIndex(NULL, xft_font, from);
}

PangoFcDecoder *
factory_callback (FcPattern *pattern, gpointer user_data)
{
  FcChar8 *family_name;

  if (FcPatternGetString (pattern, FC_FAMILY, 0, &family_name) != FcResultMatch)
    return NULL;

  printf("family name was %s\n", family_name);

  if (!strcmp(family_name, "Luxi Sans")) {
    decoder = pango_fc_decoder_test_new();
    return PANGO_FC_DECODER(decoder);
  }

  return NULL;
}
